@@ -18,12 +18,13 @@ Warning:
 """
 
 from http import HTTPStatus
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Tuple, Type, Union
 
 from .collections import HTTPHeaderDict
 from .constances import default_headers
 from .errors import (
     InsecureTransportError,
+    InvalidClientError,
     InvalidRequestError,
     MethodNotAllowedError,
     TemporarilyUnavailableError,
@@ -33,6 +34,7 @@ from .errors import (
 from .grant_type import (
     AuthorizationCodeGrantType,
     ClientCredentialsGrantType,
+    GrantTypeBase,
     PasswordGrantType,
     RefreshTokenGrantType,
 )
@@ -49,13 +51,18 @@ from .responses import (
     TokenInactiveIntrospectionResponse,
 )
 from .storage import BaseStorage
-from .types import GrantType, RequestMethod, ResponseMode, ResponseType, TokenType
+from .types import (
+    GrantType,
+    RequestMethod,
+    ResponseMode,
+    ResponseType,
+    TokenType,
+)
 from .utils import (
     build_uri,
     catch_errors_and_unavailability,
     decode_auth_headers,
     enforce_list,
-    is_secure_transport,
 )
 
 
@@ -89,11 +96,28 @@ class AuthorizationServer:
         if grant_types is not None:
             self.grant_types = grant_types
 
+    def is_secure_transport(self, request: Request) -> bool:
+        """
+        Verifies the request was sent via a protected SSL tunnel.
+
+        Note:
+            This method simply checks if the request URL contains
+            ``https://`` at the start of it. It does **not** ensure
+            if the SSL certificate is valid.
+        Args:
+            request: :py:class:`aioauth.requests.Request` object.
+        Returns:
+            Flag representing whether or not the transport is secure.
+        """
+        if request.settings.INSECURE_TRANSPORT:
+            return True
+        return request.url.lower().startswith("https://")
+
     def validate_request(self, request: Request, allowed_methods: List[RequestMethod]):
         if not request.settings.AVAILABLE:
             raise TemporarilyUnavailableError(request=request)
 
-        if not is_secure_transport(request):
+        if not self.is_secure_transport(request):
             raise InsecureTransportError(request=request)
 
         if request.method not in allowed_methods:
@@ -116,7 +140,7 @@ class AuthorizationServer:
             Below is an example utilizing FastAPI as the server framework.
         .. code-block:: python
 
-            from aioauth.fastapi.utils import to_oauth2_request, to_fastapi_response
+            from aioauth_fastapi.utils import to_oauth2_request, to_fastapi_response
 
             @app.get("/token/introspect")
             async def introspect(request: fastapi.Request) -> fastapi.Response:
@@ -135,7 +159,7 @@ class AuthorizationServer:
             response: An :py:class:`aioauth.responses.Response` object.
         """
         self.validate_request(request, [RequestMethod.POST])
-        client_id, _ = decode_auth_headers(request)
+        client_id, _ = self.get_client_credentials(request)
 
         token_types = set(TokenType)
         token_type = TokenType.REFRESH
@@ -162,8 +186,6 @@ class AuthorizationServer:
             TokenInactiveIntrospectionResponse, TokenActiveIntrospectionResponse
         ]
 
-        token_response = TokenInactiveIntrospectionResponse()
-
         if token and not token.is_expired and not token.revoked:
             token_response = TokenActiveIntrospectionResponse(
                 scope=token.scope,
@@ -171,6 +193,8 @@ class AuthorizationServer:
                 expires_in=token.expires_in,
                 token_type=token_type,
             )
+        else:
+            token_response = TokenInactiveIntrospectionResponse()
 
         content = token_response._asdict()
 
@@ -178,6 +202,22 @@ class AuthorizationServer:
             content=content, status_code=HTTPStatus.OK, headers=default_headers
         )
 
+    def get_client_credentials(self, request: Request) -> Tuple[str, str]:
+        client_id = request.post.client_id
+        client_secret = request.post.client_secret
+
+        if client_id is None or client_secret is None:
+            authorization = request.headers.get("Authorization", "")
+            headers = HTTPHeaderDict({"WWW-Authenticate": "Basic"})
+
+            # Get client credentials from the Authorization header.
+            try:
+                client_id, client_secret = decode_auth_headers(authorization)
+            except ValueError as exc:
+                raise InvalidClientError(request=request, headers=headers) from exc
+
+        return client_id, client_secret
+
     @catch_errors_and_unavailability
     async def create_token_response(self, request: Request) -> Response:
         """Endpoint to obtain an access and/or ID token by presenting an
@@ -193,7 +233,7 @@ class AuthorizationServer:
             Below is an example utilizing FastAPI as the server framework.
         .. code-block:: python
 
-            from aioauth.fastapi.utils import to_oauth2_request, to_fastapi_response
+            from aioauth_fastapi.utils import to_oauth2_request, to_fastapi_response
 
             @app.post("/token")
             async def token(request: fastapi.Request) -> fastapi.Response:
@@ -212,6 +252,7 @@ class AuthorizationServer:
             response: An :py:class:`aioauth.responses.Response` object.
         """
         self.validate_request(request, [RequestMethod.POST])
+        client_id, client_secret = self.get_client_credentials(request)
 
         if not request.post.grant_type:
             # grant_type request value is empty
@@ -219,15 +260,29 @@ class AuthorizationServer:
                 request=request, description="Request is missing grant type."
             )
 
-        GrantTypeClass = self.grant_types.get(request.post.grant_type)
+        GrantTypeClass: Type[
+            Union[
+                GrantTypeBase,
+                AuthorizationCodeGrantType,
+                PasswordGrantType,
+                RefreshTokenGrantType,
+                ClientCredentialsGrantType,
+            ]
+        ]
 
-        if GrantTypeClass is None:
-            # Requested GrantType was not found in the list of the grant_types.
-            raise UnsupportedGrantTypeError(request=request)
+        try:
+            GrantTypeClass = self.grant_types[request.post.grant_type]
+        except KeyError as exc:
+            # grant_type request value is invalid
+            raise UnsupportedGrantTypeError(request=request) from exc
 
-        grant_type = GrantTypeClass(storage=self.storage)
+        grant_type = GrantTypeClass(
+            storage=self.storage, client_id=client_id, client_secret=client_secret
+        )
 
-        response = await grant_type.create_token_response(request)
+        client = await grant_type.validate_request(request)
+
+        response = await grant_type.create_token_response(request, client)
         content = response._asdict()
 
         return Response(
@@ -304,7 +359,10 @@ class AuthorizationServer:
 
         for ResponseTypeClass in response_type_classes:
             response_type = ResponseTypeClass(storage=self.storage)
-            response = await response_type.create_authorization_response(request)
+            client = await response_type.validate_request(request)
+            response = await response_type.create_authorization_response(
+                request, client
+            )
             responses.update(response._asdict())
 
         # See: https://openid.net/specs/oauth-v2-multiple-response-types-1_0.html#Combinations
