@@ -17,12 +17,11 @@ import logging
 import random
 import string
 from base64 import b64decode, b64encode
-from typing import Callable, Dict, List, Optional, Set, Tuple, Union
+from typing import Any, Callable, Coroutine, Dict, List, Optional, Set, Tuple, Union
 from urllib.parse import quote, urlencode, urlparse, urlunsplit
 
 from .collections import HTTPHeaderDict
 from .errors import (
-    InvalidClientError,
     OAuth2Error,
     ServerError,
     TemporarilyUnavailableError,
@@ -36,24 +35,6 @@ UNICODE_ASCII_CHARACTER_SET = string.ascii_letters + string.digits
 log = logging.getLogger(__name__)
 
 
-def is_secure_transport(request: Request) -> bool:
-    """
-    Verifies the request was sent via a protected SSL tunnel.
-
-    Note:
-        This method simply checks if the request URL contains
-        ``https://`` at the start of it. It does **not** ensure
-        if the SSL certificate is valid.
-    Args:
-        request: :py:class:`aioauth.requests.Request` object.
-    Returns:
-        Flag representing whether or not the transport is secure.
-    """
-    if request.settings.INSECURE_TRANSPORT:
-        return True
-    return request.url.lower().startswith("https://")
-
-
 def get_authorization_scheme_param(
     authorization_header_value: str,
 ) -> Tuple[str, str]:
@@ -105,8 +86,6 @@ def enforce_list(scope: Optional[Union[str, List, Set, Tuple]]) -> List:
         scope: An iterable or string that contains scopes.
     Returns:
         A list of scopes.
-    Raises:
-        TypeError: The ``scope`` value passed is not of the proper type.
     """
     if isinstance(scope, (tuple, list, set)):
         return [str(s) for s in scope]
@@ -184,7 +163,7 @@ def encode_auth_headers(client_id: str, client_secret: str) -> HTTPHeaderDict:
     return HTTPHeaderDict(Authorization=f"basic {authorization.decode()}")
 
 
-def decode_auth_headers(request: Request) -> Tuple[str, str]:
+def decode_auth_headers(authorization: str) -> Tuple[str, str]:
     """
     Decodes an encrypted HTTP basic authentication string.
     Returns a tuple of the form ``(client_id, client_secret)``, and
@@ -192,29 +171,25 @@ def decode_auth_headers(request: Request) -> Tuple[str, str]:
     could be decoded.
 
     Args:
-        request: A request object.
+        authorization: Authorization header string.
     Returns:
         Tuple of the form ``(client_id, client_secret)``.
     Raises:
-        aioauth.errors.InvalidClientError: Could not be decoded.
+        ValueError: Invalid `authorization` header string.
     """
-    authorization = request.headers.get("Authorization", "")
-
-    headers = HTTPHeaderDict({"WWW-Authenticate": "Basic"})
-
     scheme, param = get_authorization_scheme_param(authorization)
     if not authorization or scheme.lower() != "basic":
-        raise InvalidClientError(request=request, headers=headers)
+        raise ValueError("Invalid authoirzation header string.")
 
     try:
         data = b64decode(param).decode("ascii")
-    except (ValueError, UnicodeDecodeError, binascii.Error):
-        raise InvalidClientError(request=request, headers=headers)
+    except (ValueError, UnicodeDecodeError, binascii.Error) as exc:
+        raise ValueError("Invalid base64 encoding.") from exc
 
     client_id, separator, client_secret = data.partition(":")
 
     if not separator:
-        raise InvalidClientError(request=request, headers=headers)
+        raise ValueError("Separator was not provided.")
 
     return client_id, client_secret
 
@@ -237,7 +212,7 @@ def create_s256_code_challenge(code_verifier: str) -> str:
     return base64.urlsafe_b64encode(data).rstrip(b"=").decode()
 
 
-def catch_errors_and_unavailability(f) -> Callable:
+def catch_errors_and_unavailability(f) -> Callable[..., Coroutine[Any, Any, Response]]:
     """
     Decorator that adds error catching to the function passed.
 
@@ -248,14 +223,14 @@ def catch_errors_and_unavailability(f) -> Callable:
     """
 
     @functools.wraps(f)
-    async def wrapper(self, request: Request, *args, **kwargs) -> Optional[Response]:
+    async def wrapper(self, request: Request, *args, **kwargs) -> Response:
         error: Union[TemporarilyUnavailableError, ServerError]
 
         try:
             response = await f(self, request, *args, **kwargs)
         except OAuth2Error as exc:
             content = ErrorResponse(error=exc.error, description=exc.description)
-            log.debug("%s %r" % (exc, request))
+            log.debug("%s %r", exc, request)
             return Response(
                 content=content._asdict(),
                 status_code=exc.status_code,
